#include "Triangle.h"
#include "glut.h"
#include <cstdlib>


//---------------------------------------------------------------------------
// Data Pool

::std::vector<CVertex> vertexPool;
::std::vector<CTriangle> trianglePool;


//---------------------------------------------------------------------------
// CVertex

void CVertex::addTriangle(unsigned int id)
{
	bool b = triangleSet_.insert(id).second;
	assert(b);
}

double CVertex::calcEdgeCollapseCost(const CVertex& vtx) const
//  vtx
{
	//    Ȯ
	assert(!pos_.is_equal(vtx.pos_));

	//  [ 11-1]  U 
	swTVector3<double> n(0.0, 0.0, 0.0);
	for (::std::set<unsigned int>::const_iterator cit = triangleSet_.begin(); cit != triangleSet_.end(); ++cit)
		n += trianglePool[*cit].getNormal();
	n /= triangleSet_.size();

	//  [ 11-2]  U  V 
	swTVector3<double> e(vtx.pos_ - pos_);

	//  [ 11-3]  n  e  
	swTVector3<double> d(n.cross(e));
	d.normalize();

	swTVector3<double> tp, tn;
	{
		::std::vector<CTriangle> tris;
		for (::std::vector<CTriangle>::iterator it = trianglePool.begin(); it != trianglePool.end(); ++it)
			if (it->hasEdge(*this, vtx)) tris.push_back(*it);

		//  ϰ ϴ  ϴ ﰢ ϳ ̻ 
		assert(tris.size() == 2);

		::std::vector<swTVector3<double> > W(2);
		for (int i = 0; i < 2; ++i) {
			CVertex& vtx1 = tris[i].getVertex(0);
			CVertex& vtx2 = tris[i].getVertex(1);
			CVertex& vtx3 = tris[i].getVertex(2);

			if (!pos_.is_equal(vtx1.pos_) && !vtx.pos_.is_equal(vtx1.pos_)) W[i] = vtx1.pos_;
			else if (!pos_.is_equal(vtx2.pos_) && !vtx.pos_.is_equal(vtx2.pos_)) W[i] = vtx2.pos_;
			else if (!pos_.is_equal(vtx3.pos_) && !vtx.pos_.is_equal(vtx3.pos_)) W[i] = vtx3.pos_;
			else assert(false);
		}

		//  [ 11-4]  UW  d 
		double prod[2];
		for (i = 0; i < 2; ++i)
			prod[i] = (W[i] - pos_).dot(d);

		//   Ҹ ϱ ؼ  ϴ  ﰢ
		//   n  e   鿡  ٸ ʿ Ͽ  
		assert(prod[0] * prod[1] < 0.0);

		if (prod[0] > 0.0) {
			tp = tris[0].getNormal();
			tn = tris[1].getNormal();
		}
		else {
			tp = tris[1].getNormal();
			tn = tris[0].getNormal();
		}
	}

	double dp = 1.0, dn = 1.0;
	double prod;
	for (cit = triangleSet_.begin(); cit != triangleSet_.end(); ++cit) {
		CTriangle& tt = trianglePool[*cit];
		for (int i = 0; i < 3; ++i) {
			if (!tt.getVertex(i).pos_.is_equal(pos_)) {
				prod = (tt.getVertex(i).pos_ - pos_).dot(d);
				if (prod > 0.0)	dp = __min(dp, tp.dot(tt.getNormal()));
				else dn = __min(dn, tn.dot(tt.getNormal()));
			}
		}
	}

	//  [ 11-5] ﰢ κ   ּҰ
	double dmin = __min(dp, dn);

	//  [ 11-6]  Һ
	return (1.0 - dmin) * e.norm();
}


//---------------------------------------------------------------------------
// CTriangle

CTriangle::CTriangle(unsigned int v1, unsigned int v2, unsigned int v3, size_t id)
: v1_(v1), v2_(v2), v3_(v3)
{
	calcNormal();

	vertexPool[v1_].addTriangle(id);
	vertexPool[v2_].addTriangle(id);
	vertexPool[v3_].addTriangle(id);
}

void CTriangle::draw() const
{
	swTVector3<double> vtx;
	glBegin(GL_TRIANGLES);
		glNormal3d(normal_.x(), normal_.y(), normal_.z());
		vtx = vertexPool[v1_].getPos();
		glVertex3d(vtx.x(), vtx.y(), vtx.z());
		vtx = vertexPool[v2_].getPos();
		glVertex3d(vtx.x(), vtx.y(), vtx.z());
		vtx = vertexPool[v3_].getPos();
		glVertex3d(vtx.x(), vtx.y(), vtx.z());
	glEnd();
}

CVertex CTriangle::getVertex(unsigned int i) const
{
	switch (i % 3) {
	case 0:	return vertexPool[v1_];
	case 1:	return vertexPool[v2_];
	case 2:	return vertexPool[v3_];
	}
}

void CTriangle::calcNormal()
{
	const swTVector3<double>& pos1 = vertexPool[v1_].getPos();
	const swTVector3<double>& pos2 = vertexPool[v2_].getPos();
	const swTVector3<double>& pos3 = vertexPool[v3_].getPos();

	normal_ = (pos2 - pos1).cross(pos3 - pos1);
	normal_.normalize();
}

bool CTriangle::hasEdge(const CVertex& vtx1, const CVertex& vtx2) const
{
	const swTVector3<double>& pos1 = vertexPool[v1_].getPos();
	const swTVector3<double>& pos2 = vertexPool[v2_].getPos();
	const swTVector3<double>& pos3 = vertexPool[v3_].getPos();

	const swTVector3<double>& p1 = vtx1.getPos();
	const swTVector3<double>& p2 = vtx2.getPos();

	return (p1.is_equal(pos1) || p1.is_equal(pos2) || p1.is_equal(pos3)) &&
		   (p2.is_equal(pos1) || p2.is_equal(pos2) || p2.is_equal(pos3));

	return false;
}
