#include "ode.h"
#include <iostream>
#include <cmath>


//     4th order Runge - Kutta method     //
void RungeKutta(void (*derivative)(double X, double* Y, double* dYdX),
				int N_var, int N_ODE, double* Y,/* double* dYdX,*/
				double initialvalue, double finalvalue, int Npanel)
{
	int i, j;
	double X, spacestep;
	double *k, *tempY, *dYdX;
	
	k = new double [ N_ODE * 4 ];
	tempY = new double [ N_var ];
	dYdX = new double [ N_ODE ];
	
	spacestep = ( finalvalue - initialvalue ) / double(Npanel);
	for ( i=0, X=initialvalue ; i<Npanel ; i++, X+=spacestep )  {
		for ( j=0 ; j<N_ODE ; j++ )  tempY[j] = Y[j*(Npanel+1)+i];
		(*derivative)( X, tempY, dYdX );
		for ( j=0 ; j<N_ODE ; j++ )  k[j*4] = dYdX[j] * spacestep;
		
		for ( j=0 ; j<N_ODE ; j++ )  tempY[j] = Y[j*(Npanel+1)+i] + k[j*4] / 2.0;
		(*derivative)( X+spacestep/2.0, tempY, dYdX );
		for ( j=0 ; j<N_ODE ; j++ )  k[j*4+1] = dYdX[j] * spacestep;
		
		for ( j=0 ; j<N_ODE ; j++ )
			tempY[j] = Y[j*(Npanel+1)+i] + ( sqrt(2.0)-1.0 ) * k[j*4] / 2.0
			+ ( 2.0-sqrt(2.0) ) * k[j*4+1] / 2.0;
		(*derivative)( X+spacestep/2.0, tempY, dYdX );
		for ( j=0 ; j<N_ODE ; j++ )  k[j*4+2] = dYdX[j] * spacestep;
		
		for ( j=0 ; j<N_ODE ; j++ )
			tempY[j] = Y[j*(Npanel+1)+i] + - sqrt(2.0) * k[j*4+1] / 2.0
			+ ( 1.0+sqrt(2.0)/2.0 ) * k[j*4+2];
		(*derivative)( X+spacestep, tempY, dYdX );
		for ( j=0 ; j<N_ODE ; j++ )  k[j*4+3] = dYdX[j] * spacestep;
		
		for ( j=0 ; j<N_ODE ; j++ )
			Y[j*(Npanel+1)+i+1] = Y[j*(Npanel+1)+i]
			+ ( k[j*4]+( 2.0-sqrt(2.0) )*k[j*4+1]
			+ ( 2.0+sqrt(2.0) )*k[j*4+2]+k[j*4+3] ) / 6.0;
	}
	
	delete [] k;
	delete [] tempY;
	delete [] dYdX;
}  /* End of RungeKutta Function */
/*
void ode2( double x, double* y, double* dydx )
{
	dydx[0] = y[1];
	dydx[1] = 0.5 * ( x + y[0] + y[1] + 2.0 );
}


void main( void )
{
	int i, Npanel=10;
	double* result = new double [ (Npanel+1) * 2 ];

	result[0] = 0.0;
	result[1*(Npanel+1)] = 0.0;
	RungeKutta( ode2, 2, 2, (double *)result, 0.0, 1.0, Npanel );
	for ( i=0 ; i<=Npanel ; i++ )
	  printf( "\n   Result(%d)   :::   %10.9f     %10.9f", i, result[i], result[1*(Npanel+1)+i] );

	cout << endl;
}
*/
